@@ -36,7 +36,7 @@ from copr.v3.pagination import next_page
 from copr_cli.helpers import cli_use_output_format
 from copr_cli.monitor import cli_monitor_parser
 from copr_cli.printers import cli_get_output_printer as get_printer
-from .util import ProgressBar, serializable
+from copr_cli.util import get_progress_callback, serializable
 from .build_config import MockProfile
 
 
@@ -62,17 +62,6 @@ BOOTSTRAP_MAP = {
     None: "default",
 }
 
-no_config_warning = """
-================= WARNING: =======================
-File '{0}' is incorrect.
-See documentation: man copr-cli.
-Any operation requiring credentials will fail!
-==================================================
-
-Hint: {1}
-
-"""
-
 try:
     input = raw_input
 except NameError:
@@ -103,7 +92,7 @@ class ActionDeprecated(argparse.Action):
         setattr(namespace, self.dest, values)
 
 
-def buildopts_from_args(args, progress_callback):
+def buildopts_from_args(args, progress_callback=None):
     """
     For all the build commands, parse the common set of build options.
     """
@@ -146,13 +135,7 @@ def requires_api_auth(func):
 
 class Commands(object):
     def __init__(self, config_path):
-        self.config_path = config_path or '~/.config/copr'
-
-        try:
-            self.config = config_from_file(self.config_path)
-        except CoprConfigException as ex:
-            sys.stderr.write(no_config_warning.format(self.config_path, ex))
-            self.config = {"copr_url": "http://copr.fedoraproject.org"}
+        self.config = config_from_file(config_path)
 
         if self.config.get("gssapi") is None:
             # Contrary to what is set in python-copr, we set GSSAPI on by
@@ -203,6 +186,16 @@ class Commands(object):
             owner = self.ownername
         return owner, name
 
+    def parse_dirname(self, name):
+        """
+        From specified <owner>/<dirname> or just <dirname> return the
+        tripled (owner, project, dirname).
+        """
+        owner, dirname = self.parse_name(name)
+        project = dirname.split(':')[0]
+        return owner, project, dirname
+
+
     def build_url(self, build_id):
         """
         Return the "generic" predictable url for build_id, which redirects
@@ -319,29 +312,34 @@ class Commands(object):
         # that the user actually has a valid credentials.
         self.client.base_proxy.auth_check()
 
+        username, projectname, project_dirname = self.parse_dirname(args.copr_repo)
+
         builds = []
         for pkg in args.pkgs:
             if os.path.exists(pkg):
-                bar = ProgressBar(max=os.path.getsize(pkg))
-                build_function = self.client.build_proxy.create_from_file
+                progress_callback = get_progress_callback(os.path.getsize(pkg))
+                buildopts = buildopts_from_args(args, progress_callback)
                 data = {"path": pkg}
-
-                # pylint: disable=function-redefined
-                def progress_callback(monitor):
-                    bar.next(n=8192)
-
                 print('Uploading package {0}'.format(pkg))
+                try:
+                    builds.append(self.client.build_proxy.create_from_file(
+                        ownername=username, projectname=projectname,
+                        project_dirname=project_dirname, buildopts=buildopts,
+                        **data))
+                finally:
+                    if progress_callback:
+                        progress_callback.finish()
             elif not urlparse(pkg).scheme:
                 raise CoprException("File {0} not found".format(pkg))
             else:
-                bar = None
-                progress_callback = None
-                build_function = self.client.build_proxy.create_from_url
+                buildopts = buildopts_from_args(args)
                 data = {"url": pkg}
+                builds.append(self.client.build_proxy.create_from_url(
+                    ownername=username, projectname=projectname,
+                    project_dirname=project_dirname, buildopts=buildopts,
+                    **data))
 
-            builds.append(self.process_build(args, build_function, data, bar=bar, progress_callback=progress_callback))
-
-        return builds
+        self.print_build_info_and_wait(builds, args)
 
     @requires_api_auth
     def action_build_pypi(self, args):
@@ -353,6 +351,7 @@ class Commands(object):
         data = {
             "pypi_package_name": args.packagename,
             "pypi_package_version": args.packageversion,
+            "spec_generator": args.spec_generator,
             "spec_template": args.spec_template,
             "python_versions": args.pythonversions,
         }
@@ -408,37 +407,38 @@ class Commands(object):
             'script': ''.join(args.script.readlines()),
         }
         for arg in ['script_chroot', 'script_builddeps',
-                    'script_resultdir']:
+                    'script_resultdir', 'script_repos']:
             data[arg] = getattr(args, arg)
         return self.process_build(args, self.client.build_proxy.create_from_custom, data)
 
-    def process_build(self, args, build_function, data, bar=None, progress_callback=None):
-        username, project_dirname = self.parse_name(args.copr_repo)
-        projectname = project_dirname.split(':')[0]
+    def print_build_info_and_wait(self, builds, args):
+        """
+        Print to stdout information about just submitted builds.  Note that our
+        users parse this non-parseable output, because we don't have issue #224
+        fixed.
+        """
 
-        try:
-            buildopts = buildopts_from_args(args, progress_callback)
-            result = build_function(ownername=username, projectname=projectname,
-                                    project_dirname=project_dirname, buildopts=buildopts, **data)
+        print("Build was added to {0}:".format(builds[0].projectname))
 
-            builds = result if type(result) == list else [result]
-            print("Build was added to {0}:".format(builds[0].projectname))
+        for build in builds:
+            print("  {0}".format(self.build_url(build.id)))
 
-            for build in builds:
-                print("  {0}".format(self.build_url(build.id)))
+        build_ids = [build.id for build in builds]
+        print("Created builds: {0}".format(" ".join(map(str, build_ids))))
 
-            build_ids = [build.id for build in builds]
-            print("Created builds: {0}".format(" ".join(map(str, build_ids))))
+        if not args.nowait:
+            self._watch_builds(build_ids)
 
-            if not args.nowait:
-                self._watch_builds(build_ids)
 
-        except CoprException as ex:
-            sys.stderr.write(str(ex) + "\n")
-            sys.exit(1)
-        finally:
-            if bar:
-                bar.finish()
+    def process_build(self, args, build_function, data):
+        username, projectname, project_dirname = self.parse_dirname(args.copr_repo)
+        buildopts = buildopts_from_args(args)
+        result = build_function(ownername=username,
+                                projectname=projectname,
+                                project_dirname=project_dirname,
+                                buildopts=buildopts, **data)
+        builds = result if type(result) == list else [result]
+        self.print_build_info_and_wait(builds, args)
 
 
     @requires_api_auth
@@ -466,8 +466,14 @@ class Commands(object):
             module_hotfixes=ON_OFF_MAP[args.module_hotfixes],
             fedora_review=args.fedora_review,
             appstream=ON_OFF_MAP[args.appstream],
+            runtime_dependencies=args.runtime_dependencies,
+            packit_forge_projects_allowed=args.packit_forge_projects_allowed,
         )
-        print("New project was successfully created.")
+
+        owner_part = username.replace('@', "g/")
+        project_url = '/'.join([self.client.config["copr_url"].strip("/"),
+                                'coprs', owner_part, copr, ""])
+        print("New project was successfully created: {0}".format(project_url))
 
     @requires_api_auth
     def action_modify_project(self, args):
@@ -492,6 +498,8 @@ class Commands(object):
             module_hotfixes=ON_OFF_MAP[args.module_hotfixes],
             fedora_review=ON_OFF_MAP[args.fedora_review],
             appstream=ON_OFF_MAP[args.appstream],
+            runtime_dependencies=args.runtime_dependencies,
+            packit_forge_projects_allowed=args.packit_forge_projects_allowed,
         )
 
     @requires_api_auth
@@ -720,6 +728,7 @@ class Commands(object):
             "package_name": args.name,
             "pypi_package_name": args.packagename,
             "pypi_package_version": args.packageversion,
+            "spec_generator": args.spec_generator,
             "spec_template": args.spec_template,
             "python_versions": args.pythonversions,
             "max_builds": args.max_builds,
@@ -897,17 +906,13 @@ class Commands(object):
         print("Package's default source was successfully reseted.")
 
     def action_build_package(self, args):
-        ownername, project_dirname = self.parse_name(args.copr_repo)
-        projectname = project_dirname.split(':')[0]
-        buildopts = buildopts_from_args(args, None)
+        ownername, projectname, project_dirname = self.parse_dirname(args.copr_repo)
+
+        buildopts = buildopts_from_args(args)
         build = self.client.package_proxy.build(ownername=ownername, projectname=projectname,
                                                 packagename=args.name, buildopts=buildopts,
                                                 project_dirname=project_dirname)
-        print("Build was added to {0}.".format(build.projectname))
-        print("Created builds: {0}".format(build.id))
-
-        if not args.nowait:
-            self._watch_builds([build.id])
+        self.print_build_info_and_wait([build], args)
 
     def action_build_module(self, args):
         """
@@ -962,6 +967,33 @@ class Commands(object):
                                                       request)
         print("success")
 
+
+def create_and_modify_common_opts(parser):
+    """
+    De-duplicate options for 'copr create' and 'copr modify' options.
+    """
+    parser.add_argument(
+        "--repo", dest="repos", action="append", metavar="REPO_BASEURL", help=(
+            "Repository that will be enabled at package build time, can be "
+            "specified multiple times."
+    ))
+    parser.add_argument(
+        "--runtime-repo-dependency", dest="runtime_dependencies",
+        metavar="REPO_BASEURL", action="append", help=(
+            "Repository that will be automatically enabled together "
+            "with repository from this copr, e.g. by `dnf copr enable`. "
+            "This can be specified multiple times."
+    ))
+
+    parser.add_argument(
+        "--packit-forge-project-allowed", dest="packit_forge_projects_allowed",
+        metavar="FORGE_PROJECT", action="append", help=(
+            "Forge project that will be allowed to build in this project "
+            "via Packit in format github.com/packit/ogr. "
+            "Can be specified multiple times."
+    ))
+
+
 def setup_parser():
     """
     Set the main arguments.
@@ -1032,8 +1064,9 @@ def setup_parser():
                                help="The name of the copr to create")
     parser_create.add_argument("--chroot", dest="chroots", action="append",
                                help="Chroot to use for this copr")
-    parser_create.add_argument("--repo", dest="repos", action="append",
-                               help="Repository to add to this copr")
+
+
+
     parser_create.add_argument("--initial-pkgs", dest="initial_pkgs",
                                action="append",
                                help="List of packages URL to build in this "
@@ -1093,6 +1126,8 @@ def setup_parser():
         help=("Generate AppStream metadata for this project. Generating "
               "metadata slows down the builds in large Copr projects."))
 
+    create_and_modify_common_opts(parser_create)
+
     parser_create.set_defaults(func="action_create")
 
     # create the parser for the "modify_project" command
@@ -1105,8 +1140,6 @@ def setup_parser():
                                help="Description of the copr")
     parser_modify.add_argument("--instructions",
                                help="Instructions for the copr")
-    parser_modify.add_argument("--repo", dest="repos", action="append",
-                               help="Repository to add to this copr")
     parser_modify.add_argument("--disable_createrepo", type=str2bool,
                                help="Disable metadata auto generation")
     parser_modify.add_argument("--enable-net", choices=["on", "off"],
@@ -1151,6 +1184,8 @@ def setup_parser():
         help=("Generate AppStream metadata for this project. Generating "
               "metadata slows down the builds in large Copr projects."))
 
+    create_and_modify_common_opts(parser_modify)
+
     parser_modify.set_defaults(func="action_modify_project")
 
     # create the parser for the "delete" command
@@ -1191,6 +1226,15 @@ def setup_parser():
                                          help="For what Python versions to build (by default: 3 2)")
     parser_pypi_args_optional.add_argument("--packageversion", metavar = "PYPIVERSION",
                                          help="Version of the PyPI package to be built (by default latest)")
+
+    parser_pypi_args_optional.add_argument(
+        "--spec-generator",
+        dest="spec_generator",
+        help="Tool for generating specfile from a PyPI package",
+        choices=["pyp2rpm", "pyp2spec"],
+        default="pyp2rpm",
+    )
+
     parser_pypi_args_optional.add_argument("--template", "-t", dest="spec_template",
                                          help="Spec template to be used to build srpm with pyp2rpm")
 
@@ -1260,6 +1304,10 @@ def setup_parser():
             '--script-resultdir',
             help='where SCRIPT generates the result, relatively to script\'s '
                  '$PWD (defaults to \'.\')')
+    parser_custom_args_parent.add_argument(
+            '--script-repos',
+            help="space separated string of additional repo urls for script dependencies"
+    )
 
     #########################################################
     ###                    Build options                  ###
